@@ -1,5 +1,5 @@
 import logging
-from typing import Optional
+from typing import Optional, Generator
 from urllib.parse import urlparse, parse_qs
 from confluence_cli.cli import ConfluenceWrapper
 
@@ -7,6 +7,54 @@ from confluence_cli.cli import ConfluenceWrapper
 logger = logging.getLogger("confluence_log")
 
 
+class CloudCQLResultIterator(object):
+
+    """
+    Pagination changes in cloud from feb-2020
+    See https://community.developer.atlassian.com/t/upcoming-changes-to-modernize-search-rest-apis/37746"""
+
+    def __init__(self, confluence: ConfluenceWrapper, cql: str, expand: str, page_size: int):
+        """Create a CQLPaginator instance with cql data
+
+        Args:
+            confluence (ConfluenceWrapper): [description]
+            cql (str): [cql string]
+            expand (str): [expand]
+            page_size (int): [page size]
+        """
+
+        self.confluence = confluence
+        self.cql = cql
+        self.expand = expand
+        self.page_size = page_size
+        self.next_params = None
+
+    def __iter__(self):
+        return self._get_results()
+
+    def _get_results(self) -> Generator:
+        resp = self.confluence.cql(cql=self.cql, expand=self.expand, limit=self.page_size)
+        contents = resp.get("results")
+        for content in contents:
+            yield content
+
+        next_params = self._get_params_from_next(resp.get("_links").get("next"))
+        while next_params:
+            resp = self.confluence.get("rest/api/search", params=next_params)
+            contents = resp.get("results")
+            for content in contents:
+                yield content
+            next_params = self._get_params_from_next(resp.get("_links").get("next"))
+
+    @staticmethod
+    def _get_params_from_next(url: str) -> Optional[dict]:
+        if not url:
+            return None
+        query = urlparse(url).query
+        dl = parse_qs(query)
+        d = {key: dl.get(key, [""])[0] for key in dl.keys()}
+        return d
+
 class CloudCQLPaginator(object):
     """
     Pagination changes in cloud from feb-2020
@@ -27,7 +75,7 @@ class CloudCQLPaginator(object):
         self.expand = expand
         self.page_size = page_size
         self.total_size: Optional[int] = None
-        self.next_params = None
+        self.next_params: Optional[dict] = None
 
     def __iter__(self):
         return self
@@ -66,8 +114,51 @@ class CloudCQLPaginator(object):
         return d
 
 
+class CQLResultIterator(object):
+    """Use like:
+
+    result_pages = []
+    for page in cql_res_iter:
+        pages.append(page)
+    """
+
+    def __init__(self, confluence: ConfluenceWrapper, cql: str, expand: str, page_size: int):
+        """Create a CQLResultIterator instance with cql data
+
+        Args:
+            confluence (ConfluenceWrapper): [description]
+            cql (str): [cql string]
+            expand (str): [expand]
+            page_size (int): [page size]
+        """
+        self.confluence = confluence
+        self.cql = cql
+        self.expand = expand
+        self.page_size = page_size
+
+    def __iter__(self):
+        return self._get_results()
+
+    def _get_results(self):
+        start = 0
+        total_size = 1
+        while start < total_size:
+            contents = self.confluence.cql(cql=self.cql, start=start, expand=self.expand, limit=self.page_size)
+            if contents:
+                # confluence wrapper sets totalSize key inside content (see confluence wrapper cql method)
+                total_size = int(contents[0].get('totalSize'))
+            else:
+                logger.info("Total Size cql: 0")
+                self.total_size = 0
+
+            for content in contents:
+                yield content
+            start = start + self.page_size
+
+
 class CQLPaginator(object):
     """Use like:
+
     result_pages = []
     for result_pages in cql_paginator:
         pages.extend(result_pages)
@@ -145,6 +236,6 @@ class PagePropertiesPaginator(object):
 
     def _get_result_page(self) -> dict:
         result = self.confluence.get_pages_detail_lines(self.cql, self.space_key, self.page_size, self.page_index)
-        self.total_pages: int = result.get("totalPages")
-        self.current_page: int = result.get("currentPage")
+        self.total_pages = result.get("totalPages")
+        self.current_page = result.get("currentPage")
         return result
